/*--------------------------------------------------------------------------*/
/*                                                                          */
/*                                                                          */
/*      ------------         Bit-Bucket Software <no-Inc>                   */
/*      \ 10001101 /         Writers and Distributors of                    */
/*       \ 011110 /          No-Cost<no-tm> Software.                       */
/*        \ 1011 /                                                          */
/*         ------            KopyRong (K) 1987.  ALL RIGHTS REVERSED.       */
/*                                                                          */
/*                                                                          */
/*                 This module was written by Bob Hartman                   */
/*               with some hacking done by Vince Perriello                  */
/*                                                                          */
/*                                                                          */
/*                       BinkleyTerm timer routines                         */
/*                                                                          */
/*                                                                          */
/*   This  software  package is being distributed WITH FULL SOURCE CODE     */
/*   with the  following  conditions:    1)  If  anything awful happens     */
/*   because  you  use    it   (or  don't  use  it),  you  accept  full     */
/*   responsibility;  2) you  don't start making tons of voice calls to     */
/*   the authors to complain or  make  suggestions  about enhancements,     */
/*   useful or otherwise;  3) you  do not reuse this code in commercial     */
/*   products without specific permission to do so  from  the  authors;     */
/*   4) If you find any problems you send  fixes  to  the  authors  for     */
/*   inclusion  in  updates;    5) You find some way  to  express  your     */
/*   appreciation  for  this  method of distribution, either by writing     */
/*   code or  application  notes,  or  just sending along a "Thank You"     */
/*   message.                                                               */
/*                                                                          */
/*   There is  copyrighted  code  in  this product.  We either wrote it     */
/*   ourselves or got  permission  to use it.  Please don't force us to     */
/*   pay a lawyer --  have some respect for our motives and don't abuse     */
/*   this "license".                                                        */
/*                                                                          */
/*                                                                          */
/*--------------------------------------------------------------------------*/

/*

	This file contains routines to implement a simple multiple
	alarm system.  The routines allow setting any number of alarms,
	and then checking if any one of them has expired.  It also allows
	adding time to an alarm.
*/


/*
 * $Log$
 */


#include "timer.h"

/*
	long timerset (t)
	unsigned int t;

	This routine returns a timer variable based on the MS-DOS
	time.  The variable returned is a long which corresponds to
	the MS-DOS time at which the timer will expire.  The variable
	need never be used once set, but to find if the timer has in
	fact expired, it will be necessary to call the timeup function.
	The expire time 't' is in hundredths of a second.

	Note: This routine as coded had a granularity of one week. I
	have put the code that implements that inside the flag 
	"HIGH_OVERHEAD". If you want it, go ahead and use it. For
	BT's purposes, (minute) granularity should suffice.

*/
long timerset (t)
unsigned int t;
{
	long l;
	int l2;
	int hours,mins,secs,ths;

#ifdef HIGH_OVERHEAD
	extern int week_day ();
#endif

	/* Get the DOS time and day of week */
	dostime (&hours,&mins,&secs,&ths);

#ifdef HIGH_OVERHEAD
	l2 = week_day ();
#endif

	/* Figure out the hundredths of a second so far this week */
	l =

#ifdef HIGH_OVERHEAD
		l2	* PER_DAY	+
		hours	* PER_HOUR	+
#endif
		mins	* PER_MINUTE	+
		secs	* PER_SECOND	+
		ths			;

	/* Add in the timer value */
	l += t;

	/* Return the alarm off time */
	return (l);
	}

/*
	int timeup (t)
	long t;

	This routine returns a 1 if the passed timer variable corresponds
	to a timer which has expired, or 0 otherwise.
*/
int timeup (t)
long t;
{
	long l;

	/* Get current time in hundredths */
	l = timerset (0);

	/* If current is less then set by more than max int, then adjust */
	if (l < (t - 65536L))
		{
#ifdef HIGH_OVERHEAD
		l += PER_WEEK;
#else
		l += PER_HOUR;
#endif
		}

	/* Return whether the current is greater than the timer variable */ 
	return ((l - t) >= 0L);
	}


/*
	long addtime (t1, t2)
	long t1;
	unsigned int t2;

	This routine adds t2 hundredths of a second to the timer variable
	in t1.  It returns a new timer variable.
*/
long addtime (t1, t2)
long t1;
unsigned int t2;
{
	return (t1 + t2);
	}
